use x11rb::connection::Connection;
use x11rb::protocol::xproto::*;
use x11rb::wrapper::ConnectionExt;
use x11rb::protocol::xproto::ConnectionExt as _;
use std::{thread, time::Duration};
use x11rb::protocol::Event;

fn main() -> Result<(), Box<dyn std::error::Error>> {
    // Подключиться к X-серверу
    let (conn, screen_num) = x11rb::connect(None)?;
    let screen = &conn.setup().roots[screen_num];

    // Получить зелёный цвет
    let cmap = screen.default_colormap;
	let white = conn.alloc_color(cmap, 65535, 65535, 65535)?.reply()?.pixel;
    let black = conn.alloc_color(cmap, 0, 0, 0)?.reply()?.pixel;

    // Параметры окна
    let size = 6;
    let x = (screen.width_in_pixels / 2 - size / 2) as i16;
    let y = (screen.height_in_pixels / 2 - size / 2) as i16;

    // Создать окно
    let win = conn.generate_id()?;
    conn.create_window(
        screen.root_depth,
        win,
        screen.root,
        x,
        y,
        size as u16,
        size as u16,
        0,
        WindowClass::INPUT_OUTPUT,
        0,
        &CreateWindowAux::new()
            .background_pixel(black)
            .override_redirect(1) // Убирает управление окном менеджером окон
            .event_mask(EventMask::EXPOSURE),
    )?;

    // Установить окно поверх всех остальных
    conn.configure_window(
        win,
        &ConfigureWindowAux::new().stack_mode(StackMode::ABOVE),
    )?;

    // Сделать окно невидимым для мыши
    conn.change_window_attributes(
        win,
        &ChangeWindowAttributesAux::new().cursor(0), // Курсор None
    )?;

    // Установить свойство "_NET_WM_STATE_ABOVE", чтобы окно всегда было поверх
    let atom_net_wm_state = conn.intern_atom(false, b"_NET_WM_STATE")?.reply()?.atom;
    let atom_net_wm_state_above = conn.intern_atom(false, b"_NET_WM_STATE_ABOVE")?.reply()?.atom;
    conn.change_property32(
        PropMode::REPLACE,
        win,
        atom_net_wm_state,
        AtomEnum::ATOM,
        &[atom_net_wm_state_above],
    )?;

    // Показать окно
    conn.map_window(win)?;
    conn.flush()?;

    // Бесконечный цикл для поддержания работы окна
    loop {
        // Принудительно поднимаем окно наверх каждые 100 мс
        conn.configure_window(
            win,
            &ConfigureWindowAux::new().stack_mode(StackMode::ABOVE),
        )?;
        conn.flush()?;

        // Ждём 100 мс
        thread::sleep(Duration::from_millis(100));

        // Обрабатываем события (чтобы программа не зависала)
        if let Some(event) = conn.poll_for_event()? {
			match event {
                Event::Expose(_) => {
                    // Рисуем белый квадрат
                    let gc = conn.generate_id()?;
                    conn.create_gc(
                        gc,
                        win,
                        &CreateGCAux::new()
                            .foreground(white)
                            .background(white),
                    )?;

                    // Рисуем залитый квадрат
                    conn.poly_fill_rectangle(
                        win,
                        gc,
                        &[Rectangle {
                            x: 1,
                            y: 1,
                            width: (size-2) as u16,
                            height: (size-2) as u16,
                        }],
                    )?;
				}
				_ => {}
			}
            // Обработка событий (если нужно)
        }
    }
}
